﻿using FarPoint.Win.Spread;
using GrapeCity.CalcEngine;
using GrapeCity.Spreadsheet;
using GrapeCity.Spreadsheet.Localization;
using System;
using System.Data;
using System.Linq;
using System.Reflection;
using System.Windows.Forms;

namespace LanguagePackages
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            SetupMenu();
            SetupSpreadsheet();
        }

        private void SetupMenu()
        {
            ToolStripMenuItem def = (ToolStripMenuItem)languagePackageToolStripMenuItem.DropDownItems.Add("English", null, onLanguageSelected);
            def.Tag = fpSpread1.AsWorkbook().WorkbookSet.LanguagePackage;
            def.Checked = true;
            foreach (PropertyInfo p in typeof(LanguagePackage).GetProperties().Where(p => p.PropertyType == typeof(LanguagePackage)))
            {
                ToolStripItem item = languagePackageToolStripMenuItem.DropDownItems.Add(p.Name, null, onLanguageSelected);
                item.Tag = typeof(LanguagePackage).GetProperty(p.Name).GetValue(def.Tag);
            }
            languagePackageToolStripMenuItem.DropDownItems.Add("-");
            ToolStripItem custom = languagePackageToolStripMenuItem.DropDownItems.Add("Custom French", null, onLanguageSelected);
            custom.Tag = CreateCustomFrenchLanguagePackage();
        }

        private void SetupSpreadsheet()
        {
            fpSpread1.AllowUserFormulas = true;
            fpSpread1.Sheets[0].ColumnHeaderRowCount = 3;
            fpSpread1.Sheets[0].RowHeaderColumnCount = 3;
            fpSpread1.Sheets[0].ColumnFooterRowCount = 3;
            fpSpread1.Sheets[0].ColumnFooterVisible = true;
            fpSpread1.Sheets[0].Cells[1, 1].Text = "Last Name";
            fpSpread1.Sheets[0].Cells[1, 2].Text = "Value";
            fpSpread1.Sheets[0].Cells[2, 1].Text = "Smith";
            fpSpread1.Sheets[0].Cells[2, 2].Value = 50;
            fpSpread1.Sheets[0].Cells[3, 1].Text = "Vil";
            fpSpread1.Sheets[0].Cells[3, 2].Value = 10;
            fpSpread1.Sheets[0].Cells[4, 1].Text = "Press";
            fpSpread1.Sheets[0].Cells[4, 2].Value = 78;
            TableView t = fpSpread1.Sheets[0].AddTable("table", 1, 1, 5, 2);
            fpSpread1.Sheets[0].ColumnHeader.Cells[1, 0].Text = "Sum of Value:";
            fpSpread1.Sheets[0].ColumnHeader.Cells[1, 0].ColumnSpan = 2;
            fpSpread1.Sheets[0].ColumnFooter.Cells[1, 0].Text = "Average of Value:";
            fpSpread1.Sheets[0].ColumnFooter.Cells[1, 0].ColumnSpan = 2;
            fpSpread1.Sheets[0].RowHeader.Cells[1, 0].Text = "STDEV of Value:";
            fpSpread1.Sheets[0].RowHeader.Cells[1, 0].ColumnSpan = 2;
            fpSpread1.Sheets[0].RowHeader.Rows[1].Height = 2 * fpSpread1.Sheets[0].RowHeader.Rows[1].Height;
            fpSpread1.AsWorkbook().Worksheets[0].ColumnHeader.Cells[1, 2].Style.NumberFormat = "0.00";
            fpSpread1.AsWorkbook().Worksheets[0].RowHeader.Cells[2, 0].Style.NumberFormat = "0.00";
            fpSpread1.AsWorkbook().Worksheets[0].ColumnFooter.Cells[1, 2].Style.NumberFormat = "0.00";
            fpSpread1.Sheets[0].ColumnHeader.Cells[1, 2].Formula = "SUM(table[Value])";
            fpSpread1.Sheets[0].ColumnFooter.Cells[1, 2].Formula = "AVERAGE(table[Value])";
            fpSpread1.Sheets[0].RowHeader.Cells[2, 0].Formula = "STDEV(table[Value])";
            fpSpread1.Sheets[0].RowHeader.Cells[2, 0].ColumnSpan = 2;
            t.TotalRowVisible = true;
            fpSpread1.ActiveSheet.Cells[0, 1].Formula = "\"Total of \"&table[[#Headers],[Value]]";
            fpSpread1.ActiveSheet.Cells[0, 1].ColumnSpan = 2;
            fpSpread1.ActiveSheet.Cells[0, 3].Formula = "table[[#Totals],[Value]]";
            fpSpread1.ActiveSheet.Cells[7, 1].ColumnSpan = 2;
            fpSpread1.ActiveSheet.Cells[7, 1].Text = "Sum of Value:";
            fpSpread1.ActiveSheet.Cells[8, 1].ColumnSpan = 2;
            fpSpread1.ActiveSheet.Cells[8, 1].Text = "Average of Value:";
            fpSpread1.ActiveSheet.Cells[9, 1].ColumnSpan = 2;
            fpSpread1.ActiveSheet.Cells[9, 1].Text = "STDEV of Value:";
            fpSpread1.ActiveSheet.Cells[7, 3].Formula = "Sheet1[[#Headers],[$C$2]]";
            fpSpread1.ActiveSheet.Cells[8, 3].Formula = "Sheet1[[#Totals],[$C$2]]";
            fpSpread1.ActiveSheet.Cells[9, 3].Formula = "Sheet1[[#RowHeaders],[$A$3]]";
            fpSpread1_Sheet1.SetFormula(0, 0, "SUM(A2:A10)");
            fpSpread1_Sheet1.SetActiveCell(0, 3);
            fpSpread1.StatusBarVisible = true;
        }

        private void onLanguageSelected(object sender, EventArgs e)
        {
            ToolStripMenuItem selected = (ToolStripMenuItem)sender;
            foreach (object item in ((ToolStripMenuItem)selected.OwnerItem).DropDownItems)
                if (item is ToolStripMenuItem)
                    ((ToolStripMenuItem)item).Checked = false;
            fpSpread1.AsWorkbook().WorkbookSet.LanguagePackage = (LanguagePackage)selected.Tag;
            selected.Checked = true;
            fpSpread1.ActiveSheet.ActiveCell.Formula = fpSpread1.ActiveSheet.ActiveCell.Formula;
        }

        private LanguagePackage CreateCustomFrenchLanguagePackage()
        {
            LanguagePackage languagePackage = new LanguagePackage("CustomFrench", "français");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ACCRINT, "INTERET.ACC");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ACCRINTM, "INTERET.ACC.MAT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ADDRESS, "ADRESSE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.AGGREGATE, "AGREGAT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.AND, "ET");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ARABIC, "CHIFFRE.ARABE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.AREAS, "ZONES");
            languagePackage.CreateFunctionAlias(BuiltinFunction.AVEDEV, "ECART.MOYEN");
            languagePackage.CreateFunctionAlias(BuiltinFunction.AVERAGE, "MOYENNE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.AVERAGEIF, "MOYENNE.SI");
            languagePackage.CreateFunctionAlias(BuiltinFunction.AVERAGEIFS, "MOYENNE.SI.ENS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BETA_DIST, "LOI.BETA.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BETA_INV, "BETA.INVERSE.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BETADIST, "LOI.BETA");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BETAINV, "BETA.INVERSE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BIN2DEC, "BINDEC");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BIN2HEX, "BINHEX");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BIN2OCT, "BINOCT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BINOM_DIST, "LOI.BINOMIALE.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BINOM_INV, "LOI.BINOMIALE.INVERSE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BINOMDIST, "LOI.BINOMIALE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BITAND, "BITET");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BITOR, "BITOU");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BITRSHIFT, "BITDECALD");
            languagePackage.CreateFunctionAlias(BuiltinFunction.BITXOR, "BITOUEXCLUSIF");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CEILING, "PLAFOND");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CEILING_MATH, "PLAFOND.MATH");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CEILING_PRECISE, "PLAFOND.PRECIS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CELL, "CELLULE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CHAR, "CAR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CHIDIST, "KHIDEUX");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CHIINV, "KHIDEUX.INVERSE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CHISQ_DIST, "LOI.KHIDEUX");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CHISQ_DIST_RT, "LOI.KHIDEUX.DROITE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CHISQ_INV, "LOI.KHIDEUX.INVERSE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CHISQ_INV_RT, "LOI.KHIDEUX.INVERSE.DROITE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CHITEST, "TEST.KHIDEUX");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CHOOSE, "CHOISIR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CLEAN, "EPURAGE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COLUMN, "COLONNE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COLUMNS, "COLONNES");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COMPLEX, "COMPLEXE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CONCATENATE, "CONCATENER");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CONFIDENCE, "INTERVALLE.CONFIANCE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CONFIDENCE_NORM, "INTERVALLE.CONFIANCE.NORMAL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CONFIDENCE_T, "INTERVALLE.CONFIANCE.STUDENT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CORREL, "COEFFICIENT.CORRELATION");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COUNT, "NB");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COUNTA, "NBVAL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COUNTBLANK, "NB.VIDE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COUNTIF, "NB.SI");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COUNTIFS, "NB.SI.ENS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COUPDAYBS, "NB.JOURS.COUPON.PREC");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COUPDAYS, "NB.JOURS.COUPONS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COUPDAYSNC, "NB.JOURS.COUPON.SUIV");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COUPNCD, "DATE.COUPON.SUIV");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COUPNUM, "NB.COUPONS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COUPPCD, "DATE.COUPON.PREC");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COVAR, "COVARIANCE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COVARIANCE_P, "COVARIANCE.PEARSON");
            languagePackage.CreateFunctionAlias(BuiltinFunction.COVARIANCE_S, "COVARIANCE.STANDARD");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CRITBINOM, "CRITERE.LOI.BINOMIALE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CUMIPMT, "CUMUL.INTER");
            languagePackage.CreateFunctionAlias(BuiltinFunction.CUMPRINC, "CUMUL.PRINCPER");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DATEVALUE, "DATEVAL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DAVERAGE, "BDMOYENNE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DAY, "JOUR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DAYS, "JOURS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DAYS360, "JOURS360");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DCOUNT, "BDNB");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DCOUNTA, "BDNBVAL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DEC2BIN, "DECBIN");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DEC2HEX, "DECHEX");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DEC2OCT, "DECOCT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DEGREES, "DEGRES");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DEVSQ, "SOMME.CARRES.ECARTS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DGET, "BDLIRE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DISC, "TAUX.ESCOMPTE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DMAX, "BDMAX");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DMIN, "BDMIN");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DOLLAR, "DEVISE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DOLLARDE, "PRIX.DEC");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DOLLARFR, "PRIX.FRAC");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DPRODUCT, "BDPRODUIT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DSTDEV, "BDECARTYPE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DSTDEVP, "BDECARTYPEP");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DSUM, "BDSOMME");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DURATION, "DUREE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DVAR, "BDVAR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.DVARP, "BDVARP");
            languagePackage.CreateFunctionAlias(BuiltinFunction.EDATE, "MOIS.DECALER");
            languagePackage.CreateFunctionAlias(BuiltinFunction.EFFECT, "TAUX.EFFECTIF");
            languagePackage.CreateFunctionAlias(BuiltinFunction.EOMONTH, "FIN.MOIS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ERF_PRECISE, "ERF.PRECIS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ERFC_PRECISE, "ERFC.PRECIS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ERROR_TYPE, "TYPE.ERREUR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.EVEN, "PAIR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.EXPON_DIST, "LOI.EXPONENTIELLE.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.EXPONDIST, "LOI.EXPONENTIELLE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.F_DIST, "LOI.F.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.F_DIST_RT, "LOI.F.DROITE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.F_INV, "INVERSE.LOI.F.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.F_INV_RT, "INVERSE.LOI.F.DROITE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FALSE, "FAUX");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FDIST, "LOI.F");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FILTERXML, "FILTRE.XML");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FIND, "TROUVE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FINDB, "TROUVERB");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FINV, "INVERSE.LOI.F");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FISHERINV, "FISHER.INVERSE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FIXED, "CTXT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FLOOR, "PLANCHER");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FLOOR_MATH, "PLANCHER.MATH");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FLOOR_PRECISE, "PLANCHER.PRECIS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FORECAST, "PREVISION");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FREQUENCY, "FREQUENCE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FTEST, "TEST.F");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FV, "VC");
            languagePackage.CreateFunctionAlias(BuiltinFunction.FVSCHEDULE, "VC.PAIEMENTS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.GAMMA_DIST, "LOI.GAMMA.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.GAMMA_INV, "LOI.GAMMA.INVERSE.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.GAMMADIST, "LOI.GAMMA");
            languagePackage.CreateFunctionAlias(BuiltinFunction.GAMMAINV, "LOI.GAMMA.INVERSE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.GAMMALN, "LNGAMMA");
            languagePackage.CreateFunctionAlias(BuiltinFunction.GAMMALN_PRECISE, "LNGAMMA.PRECIS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.GCD, "PGCD");
            languagePackage.CreateFunctionAlias(BuiltinFunction.GEOMEAN, "MOYENNE.GEOMETRIQUE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.GESTEP, "SUP.SEUIL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.GROWTH, "CROISSANCE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.HARMEAN, "MOYENNE.HARMONIQUE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.HEX2BIN, "HEXBIN");
            languagePackage.CreateFunctionAlias(BuiltinFunction.HEX2DEC, "HEXDEC");
            languagePackage.CreateFunctionAlias(BuiltinFunction.HEX2OCT, "HEXOCT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.HLOOKUP, "RECHERCHEH");
            languagePackage.CreateFunctionAlias(BuiltinFunction.HOUR, "HEURE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.HYPERLINK, "LIEN_HYPERTEXTE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.HYPGEOM_DIST, "LOI.HYPERGEOMETRIQUE.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.HYPGEOMDIST, "LOI.HYPERGEOMETRIQUE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IF, "SI");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IFERROR, "SIERREUR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IFNA, "SI.NON.DISP");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMABS, "COMPLEXE.MODULE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMAGINARY, "COMPLEXE.IMAGINAIRE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMARGUMENT, "COMPLEXE.ARGUMENT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMCONJUGATE, "COMPLEXE.CONJUGUE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMCOS, "COMPLEXE.COS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMCOSH, "COMPLEXE.COSH");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMCSC, "COMPLEXE.CSC");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMCSCH, "COMPLEXE.CSCH");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMDIV, "COMPLEXE.DIV");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMEXP, "COMPLEXE.EXP");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMLN, "COMPLEXE.LN");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMLOG10, "COMPLEXE.LOG10");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMLOG2, "COMPLEXE.LOG2");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMPOWER, "COMPLEXE.PUISSANCE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMPRODUCT, "COMPLEXE.PRODUIT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMREAL, "COMPLEXE.REEL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMSEC, "COMPLEXE.SEC");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMSECH, "COMPLEXE.SECH");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMSIN, "COMPLEXE.SIN");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMSINH, "COMPLEXE.SINH");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMSQRT, "COMPLEXE.RACINE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMSUB, "COMPLEXE.DIFFERENCE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IMSUM, "COMPLEXE.SOMME");
            languagePackage.CreateFunctionAlias(BuiltinFunction.INFO, "INFORMATIONS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.INT, "ENT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.INTERCEPT, "ORDONNEE.ORIGINE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.INTRATE, "TAUX.INTERET");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IPMT, "INTPER");
            languagePackage.CreateFunctionAlias(BuiltinFunction.IRR, "TRI");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISBLANK, "ESTVIDE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISERR, "ESTERR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISERROR, "ESTERREUR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISEVEN, "EST.PAIR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISLOGICAL, "ESTLOGIQUE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISNA, "ESTNA");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISNONTEXT, "ESTNONTEXTE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISNUMBER, "ESTNUM");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISO_CEILING, "ISO.PLAFOND");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISODD, "EST.IMPAIR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISREF, "ESTREF");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ISTEXT, "ESTTEXTE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.KURT, "KURTOSIS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LARGE, "GRANDE.VALEUR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LCM, "PPCM");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LEFT, "GAUCHE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LEFTB, "GAUCHEB");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LEN, "NBCAR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LENB, "LENB");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LINEST, "DROITEREG");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LOGEST, "LOGREG");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LOGINV, "LOI.LOGNORMALE.INVERSE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LOGNORM_DIST, "LOI.LOGNORMALE.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LOGNORM_INV, "LOI.LOGNORMALE.INVERSE.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LOGNORMDIST, "LOI.LOGNORMALE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LOOKUP, "RECHERCHE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.LOWER, "MINUSCULE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MATCH, "EQUIV");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MDETERM, "DETERMAT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MDURATION, "DUREE.MODIFIEE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MEDIAN, "MEDIANE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MID, "STXT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MINVERSE, "INVERSEMAT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MIRR, "TRIM");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MMULT, "PRODUITMAT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MODE_MULT, "MODE.MULTIPLE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MODE_SNGL, "MODE.SIMPLE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MONTH, "MOIS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MROUND, "ARRONDI.AU.MULTIPLE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MULTINOMIAL, "MULTINOMIALE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.MUNIT, "MATRICE.UNITAIRE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NEGBINOM_DIST, "LOI.BINOMIALE.NEG.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NEGBINOMDIST, "LOI.BINOMIALE.NEG");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NETWORKDAYS, "NB.JOURS.OUVRES");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NETWORKDAYS_INTL, "NB.JOURS.OUVRES.INTL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NOMINAL, "TAUX.NOMINAL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NORM_DIST, "LOI.NORMALE.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NORM_INV, "LOI.NORMALE.INVERSE.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NORM_S_DIST, "LOI.NORMALE.STANDARD.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NORM_S_INV, "LOI.NORMALE.STANDARD.INVERSE.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NORMDIST, "LOI.NORMALE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NORMINV, "LOI.NORMALE.INVERSE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NORMSDIST, "LOI.NORMALE.STANDARD");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NORMSINV, "LOI.NORMALE.STANDARD.INVERSE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NOT, "NON");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NOW, "MAINTENANT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NPER, "NPM");
            languagePackage.CreateFunctionAlias(BuiltinFunction.NPV, "VAN");
            languagePackage.CreateFunctionAlias(BuiltinFunction.OCT2BIN, "OCTBIN");
            languagePackage.CreateFunctionAlias(BuiltinFunction.OCT2DEC, "OCTDEC");
            languagePackage.CreateFunctionAlias(BuiltinFunction.OCT2HEX, "OCTHEX");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ODD, "IMPAIR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ODDFPRICE, "PRIX.PCOUPON.IRREG");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ODDFYIELD, "REND.PCOUPON.IRREG");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ODDLPRICE, "PRIX.DCOUPON.IRREG");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ODDLYIELD, "REND.DCOUPON.IRREG");
            languagePackage.CreateFunctionAlias(BuiltinFunction.OFFSET, "DECALER");
            languagePackage.CreateFunctionAlias(BuiltinFunction.OR, "OU");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PERCENTILE, "CENTILE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PERCENTILE_EXC, "CENTILE.EXCLURE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PERCENTILE_INC, "CENTILE.INCLURE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PERCENTRANK, "RANG.POURCENTAGE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PERCENTRANK_EXC, "RANG.POURCENTAGE.EXCLURE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PERCENTRANK_INC, "RANG.POURCENTAGE.INCLURE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PERMUT, "PERMUTATION");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PMT, "VPM");
            languagePackage.CreateFunctionAlias(BuiltinFunction.POISSON, "LOI.POISSON");
            languagePackage.CreateFunctionAlias(BuiltinFunction.POISSON_DIST, "LOI.POISSON.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.POWER, "PUISSANCE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PPMT, "PRINCPER");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PRICE, "PRIX.TITRE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PRICEDISC, "VALEUR.ENCAISSEMENT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PRICEMAT, "PRIX.TITRE.ECHEANCE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PROB, "PROBABILITE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PRODUCT, "PRODUIT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PROPER, "NOMPROPRE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.PV, "VA");
            languagePackage.CreateFunctionAlias(BuiltinFunction.QUARTILE_EXC, "QUARTILE.EXCLURE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.QUARTILE_INC, "QUARTILE.INCLURE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.RAND, "ALEA");
            languagePackage.CreateFunctionAlias(BuiltinFunction.RANDBETWEEN, "ALEA.ENTRE.BORNES");
            languagePackage.CreateFunctionAlias(BuiltinFunction.RANK, "RANG");
            languagePackage.CreateFunctionAlias(BuiltinFunction.RANK_AVG, "MOYENNE.RANG");
            languagePackage.CreateFunctionAlias(BuiltinFunction.RANK_EQ, "EQUATION.RANG");
            languagePackage.CreateFunctionAlias(BuiltinFunction.RATE, "TAUX");
            languagePackage.CreateFunctionAlias(BuiltinFunction.RECEIVED, "VALEUR.NOMINALE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.REPLACE, "REMPLACER, REMPLACER");
            languagePackage.CreateFunctionAlias(BuiltinFunction.REPLACEB, "REMPLACERB, REMPLACERB");
            languagePackage.CreateFunctionAlias(BuiltinFunction.RIGHT, "DROITE, DROITE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.RIGHTB, "DROITE, DROITEB");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ROMAN, "ROMAIN");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ROUND, "ARRONDI");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ROUNDDOWN, "ARRONDI.INF");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ROUNDUP, "ARRONDI.SUP");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ROW, "LIGNE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ROWS, "LIGNES");
            languagePackage.CreateFunctionAlias(BuiltinFunction.RRI, "TAUX.INT.EQUIV");
            languagePackage.CreateFunctionAlias(BuiltinFunction.RSQ, "COEFFICIENT.DETERMINATION");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SEARCH, "CHERCHE, CHERCHER");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SEARCHB, "CHERCHE, CHERCHERB");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SECOND, "SECONDE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SERIESSUM, "SOMME.SERIES");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SHEET, "FEUILLE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SIGN, "SIGNE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SKEW, "COEFFICIENT.ASYMETRIE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SLN, "AMORLIN");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SLOPE, "PENTE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SMALL, "PETITE.VALEUR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SQRT, "RACINE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SQRTPI, "RACINE.PI");
            languagePackage.CreateFunctionAlias(BuiltinFunction.STANDARDIZE, "CENTREE.REDUITE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.STDEV, "ECARTYPE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.STDEV_P, "ECARTYPE.PEARSON");
            languagePackage.CreateFunctionAlias(BuiltinFunction.STDEV_S, "ECARTYPE.STANDARD");
            languagePackage.CreateFunctionAlias(BuiltinFunction.STDEVP, "ECARTYPEP");
            languagePackage.CreateFunctionAlias(BuiltinFunction.STEYX, "ERREUR.TYPE.XY");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SUBSTITUTE, "SUBSTITUE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SUBTOTAL, "SOUS.TOTAL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SUM, "SOMME");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SUMIF, "SOMME.SI");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SUMIFS, "SOMME.SI.ENS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SUMPRODUCT, "SOMMEPROD");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SUMSQ, "SOMME.CARRES");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SUMX2MY2, "SOMME.X2MY2");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SUMX2PY2, "SOMME.X2PY2");
            languagePackage.CreateFunctionAlias(BuiltinFunction.SUMXMY2, "SOMME.XMY2");
            languagePackage.CreateFunctionAlias(BuiltinFunction.T_DIST, "LOI.STUDENT.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.T_DIST_2T, "LOI.STUDENT.BILATERALE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.T_DIST_RT, "LOI.STUDENT.DROITE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.T_INV, "LOI.STUDENT.INVERSE.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.T_INV_2T, "LOI.STUDENT.INVERSE.BILATERALE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TBILLEQ, "TAUX.ESCOMPTE.R");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TBILLPRICE, "PRIX.BON.TRESOR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TBILLYIELD, "RENDEMENT.BON.TRESOR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TDIST, "LOI.STUDENT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TEXT, "TEXTE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TIME, "TEMPS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TIMEVALUE, "TEMPSVAL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TINV, "LOI.STUDENT.INVERSE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TODAY, "AUJOURDHUI");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TREND, "TENDANCE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TRIM, "SUPPRESPACE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TRIMMEAN, "MOYENNE.REDUITE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TRUE, "VRAI");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TRUNC, "TRONQUE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.TTEST, "TEST.STUDENT");
            languagePackage.CreateFunctionAlias(BuiltinFunction.UNICHAR, "UNICAR");
            languagePackage.CreateFunctionAlias(BuiltinFunction.UPPER, "MAJUSCULE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.VALUE, "CNUM");
            languagePackage.CreateFunctionAlias(BuiltinFunction.VLOOKUP, "RECHERCHEV");
            languagePackage.CreateFunctionAlias(BuiltinFunction.WEBSERVICE, "SERVICEWEB");
            languagePackage.CreateFunctionAlias(BuiltinFunction.WEEKDAY, "JOURSEM");
            languagePackage.CreateFunctionAlias(BuiltinFunction.WEEKNUM, "NO.SEMAINE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.WEIBULL, "LOI.WEIBULL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.WEIBULL_DIST, "LOI.WEIBULL.N");
            languagePackage.CreateFunctionAlias(BuiltinFunction.WORKDAY, "SERIE.JOUR.OUVRE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.WORKDAY_INTL, "SERIE.JOUR.OUVRE.INTL");
            languagePackage.CreateFunctionAlias(BuiltinFunction.XIRR, "TRI.PAIEMENTS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.XNPV, "VAN.PAIEMENTS");
            languagePackage.CreateFunctionAlias(BuiltinFunction.XOR, "OUX");
            languagePackage.CreateFunctionAlias(BuiltinFunction.YEAR, "ANNEE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.YEARFRAC, "FRACTION.ANNEE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.YIELD, "RENDEMENT.TITRE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.YIELDDISC, "RENDEMENT.SIMPLE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.YIELDMAT, "RENDEMENT.TITRE.ECHEANCE");
            languagePackage.CreateFunctionAlias(BuiltinFunction.ZTEST, "TEST.Z");
            languagePackage.MapAlias(StructuredItemSpecifiers.All, "Tout");
            languagePackage.MapAlias(StructuredItemSpecifiers.Data, "Données");
            languagePackage.MapAlias(StructuredItemSpecifiers.Headers, "En-têtes");
            languagePackage.MapAlias(StructuredItemSpecifiers.Totals, "Totaux");
            languagePackage.MapAlias(StructuredItemSpecifiers.ThisRow, "Cette ligne");
            languagePackage.MapAlias(CalcError.Name, "#NOM?");
            languagePackage.MapAlias(CalcError.Null, "#NUL!");
            languagePackage.MapAlias(CalcError.Number, "#NOMBRE!");
            languagePackage.MapAlias(CalcError.Value, "#VALEUR!");
            languagePackage.MapAlias(CalcError.GettingData, "#CHARGEMENT_DONNEES");
            languagePackage.MapAlias(true, "VRAI");
            languagePackage.MapAlias(false, "FAUX");
            return languagePackage;
        }
    }
}
